@@ -20,7 +20,6 @@ import (
 	"go.k6.io/k6/lib"
 	"go.k6.io/k6/lib/consts"
 	"go.k6.io/k6/loader"
-	"go.k6.io/k6/metrics"
 )
 
 // A Bundle is a self-contained bundle of scripts and resources.
@@ -33,9 +32,8 @@ type Bundle struct {
 
 	BaseInitContext *InitContext
 
-	RuntimeOptions    lib.RuntimeOptions
 	CompatibilityMode lib.CompatibilityMode // parsed value
-	registry          *metrics.Registry
+	preInitState      *lib.TestPreInitState
 
 	exports map[string]goja.Callable
 }
@@ -52,9 +50,24 @@ type BundleInstance struct {
 	pgm          programWithSource
 }
 
+func (bi *BundleInstance) getCallableExport(name string) goja.Callable {
+	return bi.exports[name]
+}
+
+func (bi *BundleInstance) getExported(name string) goja.Value {
+	return bi.pgm.exports.Get(name)
+}
+
 // NewBundle creates a new bundle from a source file and a filesystem.
 func NewBundle(
 	piState *lib.TestPreInitState, src *loader.SourceData, filesystems map[string]afero.Fs,
+) (*Bundle, error) {
+	return newBundle(piState, src, filesystems, lib.Options{}, true)
+}
+
+func newBundle(
+	piState *lib.TestPreInitState, src *loader.SourceData, filesystems map[string]afero.Fs,
+	options lib.Options, updateOptions bool, // TODO: try to figure out a way to not need both
 ) (*Bundle, error) {
 	compatMode, err := lib.ValidateCompatibilityMode(piState.RuntimeOptions.CompatibilityMode.String)
 	if err != nil {
@@ -80,16 +93,16 @@ func NewBundle(
 		Source:            code,
 		Program:           pgm,
 		BaseInitContext:   NewInitContext(piState.Logger, rt, c, compatMode, filesystems, loader.Dir(src.URL)),
-		RuntimeOptions:    piState.RuntimeOptions,
+		Options:           options,
 		CompatibilityMode: compatMode,
 		exports:           make(map[string]goja.Callable),
-		registry:          piState.Registry,
+		preInitState:      piState,
 	}
-	if err = bundle.instantiate(piState.Logger, rt, bundle.BaseInitContext, 0); err != nil {
+	if err = bundle.instantiate(bundle.BaseInitContext, 0); err != nil {
 		return nil, err
 	}
 
-	err = bundle.getExports(piState.Logger, rt, true)
+	err = bundle.getExports(piState.Logger, rt, updateOptions)
 	if err != nil {
 		return nil, err
 	}
@@ -103,64 +116,25 @@ func NewBundleFromArchive(piState *lib.TestPreInitState, arc *lib.Archive) (*Bun
 		return nil, fmt.Errorf("expected bundle type 'js', got '%s'", arc.Type)
 	}
 
-	rtOpts := piState.RuntimeOptions // copy the struct from the TestPreInitState
-	if !rtOpts.CompatibilityMode.Valid {
+	if !piState.RuntimeOptions.CompatibilityMode.Valid {
 		// `k6 run --compatibility-mode=whatever archive.tar` should override
 		// whatever value is in the archive
-		rtOpts.CompatibilityMode = null.StringFrom(arc.CompatibilityMode)
+		piState.RuntimeOptions.CompatibilityMode = null.StringFrom(arc.CompatibilityMode)
 	}
-	compatMode, err := lib.ValidateCompatibilityMode(rtOpts.CompatibilityMode.String)
-	if err != nil {
-		return nil, err
-	}
-
-	c := compiler.New(piState.Logger)
-	c.Options = compiler.Options{
-		Strict:            true,
-		CompatibilityMode: compatMode,
-		SourceMapLoader:   generateSourceMapLoader(piState.Logger, arc.Filesystems),
-	}
-	pgm, _, err := c.Compile(string(arc.Data), arc.FilenameURL.String(), false)
-	if err != nil {
-		return nil, err
-	}
-	rt := goja.New()
-	initctx := NewInitContext(piState.Logger, rt, c, compatMode, arc.Filesystems, arc.PwdURL)
-
 	env := arc.Env
 	if env == nil {
 		// Older archives (<=0.20.0) don't have an "env" property
 		env = make(map[string]string)
 	}
-	for k, v := range rtOpts.Env {
+	for k, v := range piState.RuntimeOptions.Env {
 		env[k] = v
 	}
-	rtOpts.Env = env
+	piState.RuntimeOptions.Env = env
 
-	bundle := &Bundle{
-		Filename:          arc.FilenameURL,
-		Source:            string(arc.Data),
-		Program:           pgm,
-		Options:           arc.Options,
-		BaseInitContext:   initctx,
-		RuntimeOptions:    rtOpts,
-		CompatibilityMode: compatMode,
-		exports:           make(map[string]goja.Callable),
-		registry:          piState.Registry,
-	}
-
-	if err = bundle.instantiate(piState.Logger, rt, bundle.BaseInitContext, 0); err != nil {
-		return nil, err
-	}
-
-	// Grab exported objects, but avoid overwriting options, which would
-	// be initialized from the metadata.json at this point.
-	err = bundle.getExports(piState.Logger, rt, false)
-	if err != nil {
-		return nil, err
-	}
-
-	return bundle, nil
+	return newBundle(piState, &loader.SourceData{
+		Data: arc.Data,
+		URL:  arc.FilenameURL,
+	}, arc.Filesystems, arc.Options, false)
 }
 
 func (b *Bundle) makeArchive() *lib.Archive {
@@ -171,13 +145,13 @@ func (b *Bundle) makeArchive() *lib.Archive {
 		FilenameURL:       b.Filename,
 		Data:              []byte(b.Source),
 		PwdURL:            b.BaseInitContext.pwd,
-		Env:               make(map[string]string, len(b.RuntimeOptions.Env)),
+		Env:               make(map[string]string, len(b.preInitState.RuntimeOptions.Env)),
 		CompatibilityMode: b.CompatibilityMode.String(),
 		K6Version:         consts.Version,
 		Goos:              runtime.GOOS,
 	}
 	// Copy env so changes in the archive are not reflected in the source Bundle
-	for k, v := range b.RuntimeOptions.Env {
+	for k, v := range b.preInitState.RuntimeOptions.Env {
 		arc.Env[k] = v
 	}
 
@@ -185,7 +159,7 @@ func (b *Bundle) makeArchive() *lib.Archive {
 }
 
 // getExports validates and extracts exported objects
-func (b *Bundle) getExports(logger logrus.FieldLogger, rt *goja.Runtime, options bool) error {
+func (b *Bundle) getExports(logger logrus.FieldLogger, rt *goja.Runtime, updateOptions bool) error {
 	pgm := b.BaseInitContext.programs[b.Filename.String()] // this is the main script and it's always present
 	exportsV := pgm.module.Get("exports")
 	if goja.IsNull(exportsV) || goja.IsUndefined(exportsV) {
@@ -201,12 +175,12 @@ func (b *Bundle) getExports(logger logrus.FieldLogger, rt *goja.Runtime, options
 		}
 		switch k {
 		case consts.Options:
-			if !options {
+			if !updateOptions {
 				continue
 			}
 			data, err := json.Marshal(v.Export())
 			if err != nil {
-				return err
+				return fmt.Errorf("error parsing script options: %w", err)
 			}
 			dec := json.NewDecoder(bytes.NewReader(data))
 			dec.DisallowUnknownFields()
@@ -231,21 +205,24 @@ func (b *Bundle) getExports(logger logrus.FieldLogger, rt *goja.Runtime, options
 }
 
 // Instantiate creates a new runtime from this bundle.
-func (b *Bundle) Instantiate(logger logrus.FieldLogger, vuID uint64) (*BundleInstance, error) {
+func (b *Bundle) Instantiate(ctx context.Context, vuID uint64) (*BundleInstance, error) {
 	// Instantiate the bundle into a new VM using a bound init context. This uses a context with a
 	// runtime, but no state, to allow module-provided types to function within the init context.
-	vuImpl := &moduleVUImpl{runtime: goja.New()}
+	rt := goja.New()
+	vuImpl := &moduleVUImpl{
+		ctx:     ctx,
+		runtime: rt,
+	}
 	init := newBoundInitContext(b.BaseInitContext, vuImpl)
-	if err := b.instantiate(logger, vuImpl.runtime, init, vuID); err != nil {
+	if err := b.instantiate(init, vuID); err != nil {
 		return nil, err
 	}
 
-	rt := vuImpl.runtime
 	pgm := init.programs[b.Filename.String()] // this is the main script and it's always present
 	bi := &BundleInstance{
 		Runtime:      rt,
 		exports:      make(map[string]goja.Callable),
-		env:          b.RuntimeOptions.Env,
+		env:          b.preInitState.RuntimeOptions.Env,
 		moduleVUImpl: vuImpl,
 		pgm:          pgm,
 	}
@@ -295,12 +272,15 @@ func (b *Bundle) initializeProgramObject(rt *goja.Runtime, init *InitContext) pr
 	return pgm
 }
 
-func (b *Bundle) instantiate(logger logrus.FieldLogger, rt *goja.Runtime, init *InitContext, vuID uint64) (err error) {
+//nolint:funlen
+func (b *Bundle) instantiate(init *InitContext, vuID uint64) (err error) {
+	rt := init.moduleVUImpl.runtime
+	logger := init.logger
 	rt.SetFieldNameMapper(common.FieldNameMapper{})
 	rt.SetRandSource(common.NewRandSource())
 
-	env := make(map[string]string, len(b.RuntimeOptions.Env))
-	for key, value := range b.RuntimeOptions.Env {
+	env := make(map[string]string, len(b.preInitState.RuntimeOptions.Env))
+	for key, value := range b.preInitState.RuntimeOptions.Env {
 		env[key] = value
 	}
 	rt.Set("__ENV", env)
@@ -315,14 +295,28 @@ func (b *Bundle) instantiate(logger logrus.FieldLogger, rt *goja.Runtime, init *
 		Logger:      logger,
 		FileSystems: init.filesystems,
 		CWD:         init.pwd,
-		Registry:    b.registry,
+		Registry:    b.preInitState.Registry,
+		LookupEnv:   b.preInitState.LookupEnv,
 	}
 	unbindInit := b.setInitGlobals(rt, init)
-	init.moduleVUImpl.ctx = context.Background()
 	init.moduleVUImpl.initEnv = initenv
 	init.moduleVUImpl.eventLoop = eventloop.New(init.moduleVUImpl)
 	pgm := b.initializeProgramObject(rt, init)
 
+	// TODO: make something cleaner for interrupting scripts, and more unified
+	// (e.g. as a part of the event loop or RunWithPanicCatching()?
+	initCtxDone := init.moduleVUImpl.ctx.Done()
+	initDone := make(chan struct{})
+	watchDone := make(chan struct{})
+	go func() {
+		select {
+		case <-initCtxDone:
+			rt.Interrupt(init.moduleVUImpl.ctx.Err())
+		case <-initDone: // do nothing
+		}
+		close(watchDone)
+	}()
+
 	err = common.RunWithPanicCatching(logger, rt, func() error {
 		return init.moduleVUImpl.eventLoop.Start(func() error {
 			f, errRun := rt.RunProgram(b.Program)
@@ -338,6 +332,8 @@ func (b *Bundle) instantiate(logger logrus.FieldLogger, rt *goja.Runtime, init *
 			panic("Somehow a commonjs main module is not wrapped in a function")
 		})
 	})
+	close(initDone)
+	<-watchDone
 
 	if err != nil {
 		var exception *goja.Exception
