@@ -15,26 +15,10 @@ import (
 
 	"go.k6.io/k6/js/common"
 	"go.k6.io/k6/js/compiler"
-	"go.k6.io/k6/js/eventloop"
 	"go.k6.io/k6/js/modules"
-	"go.k6.io/k6/js/modules/k6"
-	"go.k6.io/k6/js/modules/k6/crypto"
-	"go.k6.io/k6/js/modules/k6/crypto/x509"
-	"go.k6.io/k6/js/modules/k6/data"
-	"go.k6.io/k6/js/modules/k6/encoding"
-	"go.k6.io/k6/js/modules/k6/execution"
-	"go.k6.io/k6/js/modules/k6/grpc"
-	"go.k6.io/k6/js/modules/k6/html"
-	"go.k6.io/k6/js/modules/k6/http"
-	"go.k6.io/k6/js/modules/k6/metrics"
-	"go.k6.io/k6/js/modules/k6/ws"
 	"go.k6.io/k6/lib"
 	"go.k6.io/k6/lib/fsext"
 	"go.k6.io/k6/loader"
-
-	"github.com/grafana/xk6-redis/redis"
-	"github.com/grafana/xk6-timers/timers"
-	expws "github.com/grafana/xk6-websockets/websockets"
 )
 
 type programWithSource struct {
@@ -62,12 +46,14 @@ type InitContext struct {
 
 	// Cache of loaded programs and files.
 	programs map[string]programWithSource
+	// merge this and the above
+	k6ModulesCache map[string]goja.Value
 
 	compatibilityMode lib.CompatibilityMode
 
 	logger logrus.FieldLogger
 
-	modules map[string]interface{}
+	moduleRegistry map[string]interface{}
 }
 
 // NewInitContext creates a new initcontext with the provided arguments
@@ -82,8 +68,11 @@ func NewInitContext(
 		programs:          make(map[string]programWithSource),
 		compatibilityMode: compatMode,
 		logger:            logger,
-		modules:           getJSModules(),
+		moduleRegistry:    getJSModules(),
+		k6ModulesCache:    make(map[string]goja.Value),
 		moduleVUImpl: &moduleVUImpl{
+			// TODO: pass a real context as we did for https://github.com/grafana/k6/pull/2800,
+			// also see https://github.com/grafana/k6/issues/2804
 			ctx:     context.Background(),
 			runtime: rt,
 		},
@@ -108,16 +97,22 @@ func newBoundInitContext(base *InitContext, vuImpl *moduleVUImpl) *InitContext {
 
 		programs:          programs,
 		compatibilityMode: base.compatibilityMode,
+		k6ModulesCache:    make(map[string]goja.Value),
 		logger:            base.logger,
-		modules:           base.modules,
+		moduleRegistry:    base.moduleRegistry,
 		moduleVUImpl:      vuImpl,
 	}
 }
 
 // Require is called when a module/file needs to be loaded by a script
-func (i *InitContext) Require(arg string) goja.Value {
+func (i *InitContext) Require(arg string) (export goja.Value) {
 	switch {
 	case arg == "k6", strings.HasPrefix(arg, "k6/"):
+		var ok bool
+		if export, ok = i.k6ModulesCache[arg]; ok {
+			return export
+		}
+		defer func() { i.k6ModulesCache[arg] = export }()
 		// Builtin or external modules ("k6", "k6/*", or "k6/x/*") are handled
 		// specially, as they don't exist on the filesystem. This intentionally
 		// shadows attempts to name your own modules this.
@@ -136,53 +131,6 @@ func (i *InitContext) Require(arg string) goja.Value {
 	}
 }
 
-type moduleVUImpl struct {
-	ctx       context.Context
-	initEnv   *common.InitEnvironment
-	state     *lib.State
-	runtime   *goja.Runtime
-	eventLoop *eventloop.EventLoop
-}
-
-func (m *moduleVUImpl) Context() context.Context {
-	return m.ctx
-}
-
-func (m *moduleVUImpl) InitEnv() *common.InitEnvironment {
-	return m.initEnv
-}
-
-func (m *moduleVUImpl) State() *lib.State {
-	return m.state
-}
-
-func (m *moduleVUImpl) Runtime() *goja.Runtime {
-	return m.runtime
-}
-
-func (m *moduleVUImpl) RegisterCallback() func(func() error) {
-	return m.eventLoop.RegisterCallback()
-}
-
-/* This is here to illustrate how to use RegisterCallback to get a promise to work with the event loop
-// TODO move this to a common function or remove before merging
-
-// MakeHandledPromise will create and promise and return it's resolve, reject methods as well wrapped in such a way that
-// it will block the eventloop from exiting before they are called even if the promise isn't resolved by the time the
-// current script ends executing
-func (m *moduleVUImpl) MakeHandledPromise() (*goja.Promise, func(interface{}), func(interface{})) {
-	callback := m.eventLoop.registerCallback()
-	p, resolve, reject := m.runtime.NewPromise()
-	return p, func(i interface{}) {
-			// more stuff
-			callback(func() { resolve(i) })
-		}, func(i interface{}) {
-			// more stuff
-			callback(func() { reject(i) })
-		}
-}
-*/
-
 func toESModuleExports(exp modules.Exports) interface{} {
 	if exp.Named == nil {
 		return exp.Default
@@ -207,7 +155,7 @@ func toESModuleExports(exp modules.Exports) interface{} {
 }
 
 func (i *InitContext) requireModule(name string) (goja.Value, error) {
-	mod, ok := i.modules[name]
+	mod, ok := i.moduleRegistry[name]
 	if !ok {
 		return nil, fmt.Errorf("unknown module: %s", name)
 	}
@@ -351,34 +299,3 @@ func (i *InitContext) allowOnlyOpenedFiles() {
 
 	alreadyOpenedFS.AllowOnlyCached()
 }
-
-func getInternalJSModules() map[string]interface{} {
-	return map[string]interface{}{
-		"k6":                         k6.New(),
-		"k6/crypto":                  crypto.New(),
-		"k6/crypto/x509":             x509.New(),
-		"k6/data":                    data.New(),
-		"k6/encoding":                encoding.New(),
-		"k6/execution":               execution.New(),
-		"k6/experimental/redis":      redis.New(),
-		"k6/experimental/websockets": &expws.RootModule{},
-		"k6/experimental/timers":     timers.New(),
-		"k6/net/grpc":                grpc.New(),
-		"k6/html":                    html.New(),
-		"k6/http":                    http.New(),
-		"k6/metrics":                 metrics.New(),
-		"k6/ws":                      ws.New(),
-	}
-}
-
-func getJSModules() map[string]interface{} {
-	result := getInternalJSModules()
-	external := modules.GetJSModules()
-
-	// external is always prefixed with `k6/x`
-	for k, v := range external {
-		result[k] = v
-	}
-
-	return result
-}
