@@ -42,7 +42,7 @@ func TestVUHandleRace(t *testing.T) {
 	var getVUCount int64
 	var returnVUCount int64
 	getVU := func() (lib.InitializedVU, error) {
-		return runner.NewVU(uint64(atomic.AddInt64(&getVUCount, 1)), 0, nil)
+		return runner.NewVU(ctx, uint64(atomic.AddInt64(&getVUCount, 1)), 0, nil)
 	}
 
 	returnVU := func(_ lib.InitializedVU) {
@@ -134,7 +134,7 @@ func TestVUHandleStartStopRace(t *testing.T) {
 
 	getVU := func() (lib.InitializedVU, error) {
 		returned = make(chan struct{})
-		return runner.NewVU(atomic.AddUint64(&vuID, 1), 0, nil)
+		return runner.NewVU(ctx, atomic.AddUint64(&vuID, 1), 0, nil)
 	}
 
 	returnVU := func(v lib.InitializedVU) {
@@ -196,7 +196,7 @@ type handleVUTest struct {
 }
 
 func (h *handleVUTest) getVU() (lib.InitializedVU, error) {
-	return h.runner.NewVU(uint64(atomic.AddUint32(&h.getVUCount, 1)), 0, nil)
+	return h.runner.NewVU(context.Background(), uint64(atomic.AddUint32(&h.getVUCount, 1)), 0, nil)
 }
 
 func (h *handleVUTest) returnVU(_ lib.InitializedVU) {
@@ -371,7 +371,7 @@ func BenchmarkVUHandleIterations(b *testing.B) {
 		return nil
 	}
 	getVU := func() (lib.InitializedVU, error) {
-		return runner.NewVU(uint64(atomic.AddUint32(&getVUCount, 1)), 0, nil)
+		return runner.NewVU(context.Background(), uint64(atomic.AddUint32(&getVUCount, 1)), 0, nil)
 	}
 
 	returnVU := func(_ lib.InitializedVU) {
