@@ -21,7 +21,6 @@ import (
 	"github.com/stretchr/testify/require"
 	"golang.org/x/net/http2"
 
-	"go.k6.io/k6/lib"
 	"go.k6.io/k6/lib/netext"
 	"go.k6.io/k6/lib/testutils/httpmultibin"
 	"go.k6.io/k6/lib/types"
@@ -216,7 +215,12 @@ func TestX509HostnameError(t *testing.T) {
 	}
 	var err error
 	badHostname := "somewhere.else"
-	tb.Dialer.Hosts[badHostname], err = lib.NewHostAddress(net.ParseIP(tb.Replacer.Replace("HTTPSBIN_IP")), "")
+	badHost, err := types.NewHost(net.ParseIP(tb.Replacer.Replace("HTTPSBIN_IP")), "")
+	require.NoError(t, err)
+
+	tb.Dialer.Hosts, err = types.NewHosts(map[string]types.Host{
+		badHostname: *badHost,
+	})
 	require.NoError(t, err)
 	req, err := http.NewRequestWithContext(context.Background(), "GET", tb.Replacer.Replace("https://"+badHostname+":HTTPSBIN_PORT/get"), nil)
 	require.NoError(t, err)
@@ -347,7 +351,7 @@ func getHTTP2ServerWithCustomConnContext(t *testing.T) *httpmultibin.HTTPMultiBi
 	require.NoError(t, err)
 	http2IP := net.ParseIP(http2URL.Hostname())
 	require.NotNil(t, http2IP)
-	http2DomainValue, err := lib.NewHostAddress(http2IP, "")
+	http2DomainValue, err := types.NewHost(http2IP, "")
 	require.NoError(t, err)
 
 	// Set up the dialer with shorter timeouts and the custom domains
@@ -356,9 +360,10 @@ func getHTTP2ServerWithCustomConnContext(t *testing.T) *httpmultibin.HTTPMultiBi
 		KeepAlive: 10 * time.Second,
 		DualStack: true,
 	}, netext.NewResolver(net.LookupIP, 0, types.DNSfirst, types.DNSpreferIPv4))
-	dialer.Hosts = map[string]*lib.HostAddress{
-		http2Domain: http2DomainValue,
-	}
+	dialer.Hosts, err = types.NewHosts(map[string]types.Host{
+		http2Domain: *http2DomainValue,
+	})
+	require.NoError(t, err)
 
 	transport := &http.Transport{
 		DialContext:     dialer.DialContext,
