@@ -8,7 +8,6 @@ import (
 	"fmt"
 	"net"
 	"reflect"
-	"strconv"
 
 	"go.k6.io/k6/lib/types"
 	"go.k6.io/k6/metrics"
@@ -21,6 +20,7 @@ import (
 const DefaultScenarioName = "default"
 
 // DefaultSummaryTrendStats are the default trend columns shown in the test summary output
+//
 //nolint:gochecknoglobals
 var DefaultSummaryTrendStats = []string{"avg", "min", "med", "max", "p(90)", "p(95)"}
 
@@ -204,84 +204,6 @@ func (ipnet *IPNet) MarshalText() ([]byte, error) {
 	return []byte(ipnet.String()), nil
 }
 
-// HostAddress stores information about IP and port
-// for a host.
-type HostAddress net.TCPAddr
-
-// NewHostAddress creates a pointer to a new address with an IP object.
-func NewHostAddress(ip net.IP, portString string) (*HostAddress, error) {
-	var port int
-	if portString != "" {
-		var err error
-		if port, err = strconv.Atoi(portString); err != nil {
-			return nil, err
-		}
-	}
-
-	return &HostAddress{
-		IP:   ip,
-		Port: port,
-	}, nil
-}
-
-// String converts a HostAddress into a string.
-func (h *HostAddress) String() string {
-	return (*net.TCPAddr)(h).String()
-}
-
-// MarshalText implements the encoding.TextMarshaler interface.
-// The encoding is the same as returned by String, with one exception:
-// When len(ip) is zero, it returns an empty slice.
-func (h *HostAddress) MarshalText() ([]byte, error) {
-	if h == nil || len(h.IP) == 0 {
-		return []byte(""), nil
-	}
-
-	if len(h.IP) != net.IPv4len && len(h.IP) != net.IPv6len {
-		return nil, &net.AddrError{Err: "invalid IP address", Addr: h.IP.String()}
-	}
-
-	return []byte(h.String()), nil
-}
-
-// UnmarshalText implements the encoding.TextUnmarshaler interface.
-// The IP address is expected in a form accepted by ParseIP.
-func (h *HostAddress) UnmarshalText(text []byte) error {
-	if len(text) == 0 {
-		return &net.ParseError{Type: "IP address", Text: "<nil>"}
-	}
-
-	ip, port, err := splitHostPort(text)
-	if err != nil {
-		return err
-	}
-
-	nh, err := NewHostAddress(ip, port)
-	if err != nil {
-		return err
-	}
-
-	*h = *nh
-
-	return nil
-}
-
-func splitHostPort(text []byte) (net.IP, string, error) {
-	host, port, err := net.SplitHostPort(string(text))
-	if err != nil {
-		// This error means that there is no port.
-		// Make host the full text.
-		host = string(text)
-	}
-
-	ip := net.ParseIP(host)
-	if ip == nil {
-		return nil, "", &net.ParseError{Type: "IP address", Text: host}
-	}
-
-	return ip, port, nil
-}
-
 // ParseCIDR creates an IPNet out of a CIDR string
 func ParseCIDR(s string) (*IPNet, error) {
 	_, ipnet, err := net.ParseCIDR(s)
@@ -363,7 +285,7 @@ type Options struct {
 	BlockedHostnames types.NullHostnameTrie `json:"blockHostnames" envconfig:"K6_BLOCK_HOSTNAMES"`
 
 	// Hosts overrides dns entries for given hosts
-	Hosts map[string]*HostAddress `json:"hosts" envconfig:"K6_HOSTS"`
+	Hosts types.NullHosts `json:"hosts" envconfig:"K6_HOSTS"`
 
 	// Disable keep-alive connections
 	NoConnectionReuse null.Bool `json:"noConnectionReuse" envconfig:"K6_NO_CONNECTION_REUSE"`
@@ -413,9 +335,10 @@ type Options struct {
 // Returns the result of overwriting any fields with any that are set on the argument.
 //
 // Example:
-//   a := Options{VUs: null.IntFrom(10)}
-//   b := Options{VUs: null.IntFrom(5)}
-//   a.Apply(b) // Options{VUs: null.IntFrom(5)}
+//
+//	a := Options{VUs: null.IntFrom(10)}
+//	b := Options{VUs: null.IntFrom(5)}
+//	a.Apply(b) // Options{VUs: null.IntFrom(5)}
 func (o Options) Apply(opts Options) Options {
 	if opts.Paused.Valid {
 		o.Paused = opts.Paused
@@ -520,7 +443,7 @@ func (o Options) Apply(opts Options) Options {
 	if opts.BlockedHostnames.Valid {
 		o.BlockedHostnames = opts.BlockedHostnames
 	}
-	if opts.Hosts != nil {
+	if opts.Hosts.Valid {
 		o.Hosts = opts.Hosts
 	}
 	if opts.NoConnectionReuse.Valid {
