@@ -21,6 +21,9 @@ type DialContexter interface {
 }
 
 // State provides the volatile state for a VU.
+//
+// TODO: rename to VUState or, better yet, move to some other Go package outside
+// of lib/, where it's more obvious that this is the VU state.
 type State struct {
 	// Global options and built-in metrics.
 	//
@@ -59,7 +62,11 @@ type State struct {
 
 	VUID, VUIDGlobal uint64
 	Iteration        int64
-	Tags             *TagMap
+
+	// TODO: rename this field with one more representative
+	// because it includes now also the metadata.
+	Tags *VUStateTags
+
 	// These will be assigned on VU activation.
 	// Returns the iteration number of this VU in the current scenario.
 	GetScenarioVUIter func() uint64
@@ -73,68 +80,41 @@ type State struct {
 	GetScenarioGlobalVUIter func() uint64
 }
 
-// CloneTags makes a copy of the tags map and returns it.
-func (s *State) CloneTags() map[string]string {
-	return s.Tags.Clone()
+// VUStateTags wraps the current VU's tags and ensures a thread-safe way to
+// access and modify them exists. This is necessary because the VU tags and
+// metadata can be modified from the JS scripts via the `vu.tags` API in the
+// `k6/execution` built-in module.
+type VUStateTags struct {
+	mutex       sync.RWMutex
+	tagsAndMeta *metrics.TagsAndMeta
 }
 
-// TagMap is a safe-concurrent Tags lookup.
-type TagMap struct {
-	m     map[string]string
-	mutex sync.RWMutex
-}
-
-// NewTagMap creates a TagMap,
-// if a not-nil map is passed then it will be used as the internal map
-// otherwise a new one will be created.
-func NewTagMap(m map[string]string) *TagMap {
-	if m == nil {
-		m = make(map[string]string)
+// NewVUStateTags initializes a new VUStateTags and returns it. It's important
+// that tags is not nil and initialized via metrics.Registry.RootTagSet().
+func NewVUStateTags(tags *metrics.TagSet) *VUStateTags {
+	if tags == nil {
+		panic("the metrics.TagSet must be initialized for creating a new lib.VUStateTags")
 	}
-	return &TagMap{
-		m:     m,
+	return &VUStateTags{
 		mutex: sync.RWMutex{},
+		tagsAndMeta: &metrics.TagsAndMeta{
+			Tags: tags,
+			// metadata is intentionally nil by default
+		},
 	}
 }
 
-// Set sets a Tag.
-func (tg *TagMap) Set(k, v string) {
+// GetCurrentValues returns the current value of the VU tags and a copy of the
+// metadata (if any) in a thread-safe way.
+func (tg *VUStateTags) GetCurrentValues() metrics.TagsAndMeta {
+	tg.mutex.RLock()
+	defer tg.mutex.RUnlock()
+	return tg.tagsAndMeta.Clone()
+}
+
+// Modify allows the thread-safe modification of the current VU tags and metadata.
+func (tg *VUStateTags) Modify(callback func(tagsAndMeta *metrics.TagsAndMeta)) {
 	tg.mutex.Lock()
 	defer tg.mutex.Unlock()
-	tg.m[k] = v
-}
-
-// Get returns the Tag value and true
-// if the provided key has been found.
-func (tg *TagMap) Get(k string) (string, bool) {
-	tg.mutex.RLock()
-	defer tg.mutex.RUnlock()
-	v, ok := tg.m[k]
-	return v, ok
-}
-
-// Len returns the number of the set keys.
-func (tg *TagMap) Len() int {
-	tg.mutex.RLock()
-	defer tg.mutex.RUnlock()
-	return len(tg.m)
-}
-
-// Delete deletes a map's item based on the provided key.
-func (tg *TagMap) Delete(k string) {
-	tg.mutex.Lock()
-	defer tg.mutex.Unlock()
-	delete(tg.m, k)
-}
-
-// Clone returns a map with the entire set of items.
-func (tg *TagMap) Clone() map[string]string {
-	tg.mutex.RLock()
-	defer tg.mutex.RUnlock()
-
-	tags := make(map[string]string, len(tg.m))
-	for k, v := range tg.m {
-		tags[k] = v
-	}
-	return tags
+	callback(tg.tagsAndMeta)
 }
