@@ -17,7 +17,6 @@ var (
 
 type Sink interface {
 	Add(s Sample)                              // Add a sample to the sink.
-	Calc()                                     // Make final calculations.
 	Format(t time.Duration) map[string]float64 // Data for thresholds.
 	IsEmpty() bool                             // Check if the Sink is empty.
 }
@@ -37,8 +36,6 @@ func (c *CounterSink) Add(s Sample) {
 // IsEmpty indicates whether the CounterSink is empty.
 func (c *CounterSink) IsEmpty() bool { return c.First.IsZero() }
 
-func (c *CounterSink) Calc() {}
-
 func (c *CounterSink) Format(t time.Duration) map[string]float64 {
 	return map[string]float64{
 		"count": c.Value,
@@ -66,20 +63,17 @@ func (g *GaugeSink) Add(s Sample) {
 	}
 }
 
-func (g *GaugeSink) Calc() {}
-
 func (g *GaugeSink) Format(t time.Duration) map[string]float64 {
 	return map[string]float64{"value": g.Value}
 }
 
 type TrendSink struct {
-	Values  []float64
-	jumbled bool
+	Values []float64
+	sorted bool
 
 	Count    uint64
 	Min, Max float64
 	Sum, Avg float64
-	Med      float64
 }
 
 // IsEmpty indicates whether the TrendSink is empty.
@@ -87,7 +81,7 @@ func (t *TrendSink) IsEmpty() bool { return t.Count == 0 }
 
 func (t *TrendSink) Add(s Sample) {
 	t.Values = append(t.Values, s.Value)
-	t.jumbled = true
+	t.sorted = false
 	t.Count += 1
 	t.Sum += s.Value
 	t.Avg = t.Sum / float64(t.Count)
@@ -108,10 +102,14 @@ func (t *TrendSink) P(pct float64) float64 {
 	case 1:
 		return t.Values[0]
 	default:
+		if !t.sorted {
+			sort.Float64s(t.Values)
+			t.sorted = true
+		}
+
 		// If percentile falls on a value in Values slice, we return that value.
 		// If percentile does not fall on a value in Values slice, we calculate (linear interpolation)
 		// the value that would fall at percentile, given the values above and below that percentile.
-		t.Calc()
 		i := pct * (float64(t.Count) - 1.0)
 		j := t.Values[int(math.Floor(i))]
 		k := t.Values[int(math.Ceil(i))]
@@ -120,30 +118,13 @@ func (t *TrendSink) P(pct float64) float64 {
 	}
 }
 
-func (t *TrendSink) Calc() {
-	if !t.jumbled {
-		return
-	}
-
-	sort.Float64s(t.Values)
-	t.jumbled = false
-
-	// The median of an even number of values is the average of the middle two.
-	if (t.Count & 0x01) == 0 {
-		t.Med = (t.Values[(t.Count/2)-1] + t.Values[(t.Count/2)]) / 2
-	} else {
-		t.Med = t.Values[t.Count/2]
-	}
-}
-
 func (t *TrendSink) Format(tt time.Duration) map[string]float64 {
-	t.Calc()
 	// TODO: respect the summaryTrendStats for REST API
 	return map[string]float64{
 		"min":   t.Min,
 		"max":   t.Max,
 		"avg":   t.Avg,
-		"med":   t.Med,
+		"med":   t.P(0.5),
 		"p(90)": t.P(0.90),
 		"p(95)": t.P(0.95),
 	}
@@ -164,8 +145,6 @@ func (r *RateSink) Add(s Sample) {
 	}
 }
 
-func (r RateSink) Calc() {}
-
 func (r RateSink) Format(t time.Duration) map[string]float64 {
 	var rate float64
 	if r.Total > 0 {
@@ -184,8 +163,6 @@ func (d DummySink) Add(s Sample) {
 	panic(errors.New("you can't add samples to a dummy sink"))
 }
 
-func (d DummySink) Calc() {}
-
 func (d DummySink) Format(t time.Duration) map[string]float64 {
 	return map[string]float64(d)
 }
